unit uJCompressZip;

interface

uses SysUtils, Classes, uJCompression, ZipMstr;

type

  TJZip = class(TInterfacedObject, IJCompressible)
  private
    _CompressMode : TJCompressMode;
    _ZipMstr : TZipMaster;
    Function GetCurrentMode() : TJCompressMode;
    procedure ResetZipMstr();
  public
    constructor Create();
    destructor destroy(); override;
    //interface functions
    procedure CompressDirectory(Directory : String; FileName : TFileName; Filter : TStrings);
    procedure CompressSingleFile(NewFile : TFileName; OutDir : String; CompLevel : Integer = 9);
    Procedure GetContentsOfFile(var Contents : TStrings; FileName : TFileName);
    procedure RenameContentToFile(FileName : TFileName);
    procedure RenameFileToContent(FileName : TFileName);
    Function  SupportsFileName(Input : TFileName) : Boolean; //based on full filename
    procedure UnCompressSingleFile(input : TFileName; OutDir : String; ExtrFile : TFileName);
    Function  UnCompressROM(var ZipContents : TStringList; Zip : TFileName) : String;
    Procedure UnCompressFile(input : TFileName; OutDir : String; Opt_Diff, Opt_Path, Opt_OW : boolean);
    Function  VerifyFile(Z_File : TFileName) : Boolean;
  end;

implementation

uses JCLStrings, JCLSysInfo;

constructor TJZip.Create();
begin
  inherited;
  _ZipMstr := TZipMaster.Create(nil);
  _ZipMstr.Load_Unz_Dll;
  _ZipMstr.Load_Zip_Dll;
  
  //activate silent mode
  _ZipMstr.Unattended := True;

  //we need to do some kind of test to see if the zip dlls have loaded ok.
  _CompressMode := cmFull;
end;

destructor TJZip.destroy();
begin
  _ZipMstr.Unload_Unz_Dll;
  _ZipMstr.Unload_Zip_Dll;
  FreeAndNil(_ZipMstr);
  inherited;
end;

Function TJZip.GetCurrentMode() : TJCompressMode;
begin
  Result := _CompressMode;
end;

procedure TJZip.ResetZipMstr();
begin
  _ZipMstr.rootDir := '';
  _ZipMstr.FSpecArgs.Clear;
  //reset back to default *.* behaviour, I dont know if this is needed really..
  _ZipMstr.FSpecArgs.Add('*.*');
  _ZipMstr.ZipFileName := '';
end;

{------------------------------------------------------------------------------}
{                   Public Routines                                            }
{------------------------------------------------------------------------------}

procedure TJZip.CompressDirectory(Directory : String; FileName : TFileName; Filter : TStrings);
var
  OldSettings : AddOpts;
begin
 //copy existing settings to a temp variable to reset later.
  OldSettings := _ZipMstr.AddOptions;
  try

    _ZipMstr.ZipFileName := FileName;
    //set up the zip component to the way we need it.
    _ZipMstr.RootDir := Directory;
    _ZipMstr.AddOptions := _ZipMstr.AddOptions + [AddForceDest];
    _ZipMstr.AddOptions := _ZipMstr.AddOptions + [AddRecurseDirs];
    _ZipMstr.AddOptions := _ZipMstr.AddOptions + [AddDirNames];
    _ZipMstr.AddOptions := _ZipMstr.AddOptions + [AddSeparateDirs];

    _ZipMstr.FSpecArgs.Clear;
    _ZipMstr.FSpecArgs.AddStrings(Filter);
    _ZipMstr.AddCompLevel := 9;

    try
      _ZipMstr.Add;
    except
      Raise EJCompressException.Create('Error compressing Data : ' + IntToStr(_ZipMstr.FullErrCode));
    end;

  finally
    //remember to turn off the options we set above.
    ResetZipMstr();
    _ZipMstr.AddOptions := OldSettings;
  end;
end;

{------------------------------------------------------------------------------}

procedure TJZip.CompressSingleFile(NewFile : TFileName; OutDir : String; CompLevel : Integer = 9);
Var
  Temp : String;
begin
  try
    temp := ExtractFileName(NewFile);
    Temp := ChangeFileExt(Temp, '');
    temp := OutDir + '\' + Temp + '.zip';
    _ZipMstr.ZipFileName := temp;
    _ZipMstr.FSpecArgs.Add(NewFile);
    _ZipMstr.AddCompLevel := CompLevel;

    try
      _ZipMstr.Add;
    except
      Raise EJCompressException.Create('Error in Add; Fatal DLL Exception in mainunit');
    end;
  finally
    ResetZipMstr();
  end;
end;

{------------------------------------------------------------------------------}

Procedure TJZip.GetContentsOfFile(var Contents : TStrings; FileName : TFileName);
var
  i : Integer;
begin
  try
    _ZipMstr.ZipFileName := FileName;
    Contents.Clear;
    for i := 0 to _ZipMstr.Count-1 do
      Contents.Add(_ZipMstr.DirEntry[i].FileName);
  finally
    ResetZipMstr();
  end;
end;

{------------------------------------------------------------------------------}

procedure TJZip.RenameContentToFile(FileName : TFileName);
Var
  RenRec : pZipRenameRec;
  ZipRenameList : TList;
  TmpZip : ^ZipDirEntry;
  Path, OldName, NewName : String;
  RError, i : Integer;
Begin

  try
  _ZipMstr.ZipFileName := FileName;

  if _ZipMstr.zipcontents.count = 1 then
  begin

    TmpZip := _ZipMstr.ZipContents.Items[0];

    OldName := TmpZip^.FileName;

    Path := ExtractFileExt(OldName);

    NewName := ChangeFileExt(ExtractFileName(FileName), Path);

    If StrCompare(ChangeFileExt(OldName,''), ChangeFileExt(NewName,'')) <> 0  then
    Begin
      New(RenRec);
      Try
        ZipRenameList := TList.Create;

        RenRec^.Source := OldName;
        RenRec^.Dest := NewName;
        RenRec^.DateTime := 0;

        ZipRenameList.Add(RenRec);

        RError := _ZipMstr.Rename(ZipRenameList, 0);

        if RError <> 0  then
          raise EJCompressException.Create('Error ' + IntToStr( RError ) + ' occurred in rename zip specification(s)');

      Finally
        for i := 0 to ZipRenameList.Count - 1 do
        begin
          RenRec := ZipRenameList.Items[i];
          Dispose( RenRec );
        end;

        FreeAndNil(ZipRenameList);
      end;

    End;

  end;

  finally
    ResetZipMstr();
  end;
End;

{------------------------------------------------------------------------------}

procedure TJZip.RenameFileToContent(FileName : TFileName);
Var
  Name : String;
  TmpZip : ^ZipDirEntry;
Begin
  try
    _ZipMstr.ZipFileName := FileName;

    if _ZipMstr.zipcontents.count = 1 then
    begin
      TmpZip := _ZipMstr.ZipContents.Items[0];

      name := TmpZip^.FileName;
      name := ChangeFileExt(name,'.zip');
      if not renameFile(FileName, name) then
        raise EJCompressException.Create('Error renaming file to' + Name);
    end
    else
      Raise EJCompressException.Create('Zip contained more than 1 file');
  finally
    ResetZipMstr();
  end;
End;

{------------------------------------------------------------------------------}

Function  TJZip.SupportsFileName(Input : TFileName) : Boolean; //based on full filename
begin
  try
    if JCLStrings.StrCompare(ExtractFileExt(Input), '.zip') =0 then
      Result := True
    else
      Result := False;
  except
    //if something goes wrong, return false - better safe than sorry
    Result := False;
  end;
end;

{------------------------------------------------------------------------------}

Procedure TJZip.UnCompressFile(input : TFileName; OutDir : String; Opt_Diff, Opt_Path, Opt_OW : boolean);
var
  temp : string;
begin
  try
    _ZipMstr.ZipFileName := input;

    with _ZipMstr do
    begin

      if Opt_Diff = true then
      begin
        temp := ExtractFileName(input);
        Temp := ChangeFileExt(Temp, '');
        temp := IncludeTrailingPathDelimiter(OutDir) + Temp;

        if (Not DirectoryExists(Temp)) then
          CreateDir(temp);

        ExtrBaseDir := IncludeTrailingPathDelimiter(temp);
      end
      else
        ExtrBaseDir := IncludeTrailingPathDelimiter(OutDir);
                                             
      SetCurrentDir(ExtrBaseDir);

      If Opt_OW Then
        ExtrOptions:=ExtrOptions+[ExtrOverwrite]
      else
        ExtrOptions:=ExtrOptions-[ExtrOverwrite];

      If Opt_Path Then
        ExtrOptions:=ExtrOptions+[ExtrDirNames]
      else
        ExtrOptions:=ExtrOptions-[ExtrDirNames];

      FSpecArgs.Add('*.*');

      Try
        Extract;
      except
        Raise EJCompressException.Create('Error in Extract; DLL Error : ' + IntToStr(_ZipMstr.ErrCode));
      end;
    end;

  finally
    ResetZipMstr();
  end;
end;


{------------------------------------------------------------------------------}

Function  TJZip.UnCompressROM(var ZipContents : TStringList; Zip : TFileName) : String;
var
  i : Integer;
  UseDir, OldCurrent : String;
begin

  Result := '';

  {If _ZipMstr.Settings.AllowWrite then
    UseDir := MainFrm.Settings.QPDirectory + 'temp\'
  else
    UseDir := ExtractFilePath(Zip);   }

  //now we will test if we can actually extract this file
  OldCurrent := GetCurrentDir();
  SetCurrentDir(UseDir);
  try
    If DiskFree(0) = 0 then
      UseDir := IncludeTrailingPathDelimiter(GetWindowsTempFolder()) + 'qp-temp\';
  finally
    SetCurrentDir(OldCurrent);
  end;

  {$IFDEF DEBUG}
  AddToLog('ROM will be extracted to : ' + UseDir);
  {$ENDIF}

  //now create the temp directory again.
  If not DirectoryExists(UseDir) then
    ForceDirectories(UseDir);

  try
    UncompressFile(Zip, UseDir, False, False, true);
    
    For i := 0 to _ZipMstr.Count-1 do
      ZipContents.Add(UseDir + ExtractFileName(_ZipMstr.DirEntry[i].FileName));

    //if one file was extracted this is the rom, save the path
    If _ZipMstr.Count = 1 then
      Result := UseDir + ExtractFileName(_ZipMstr.DirEntry[0].FileName);

    //if more than one file is extracted we have a problem!
    //Just try to GUESS which file is the rom... pants...
    If _ZipMstr.Count > 1 then
      For i := 0 to _ZipMstr.Count-1 do
        If (ExtractFileExt(_ZipMstr.DirEntry[i].FileName) <> '.txt') or
           (ExtractFileExt(_ZipMstr.DirEntry[i].FileName) <> '.nfo') or
           (ExtractFileExt(_ZipMstr.DirEntry[i].FileName) <> '.diz') then
        Begin
          Result := UseDir + ExtractFileName(_ZipMstr.DirEntry[i].FileName);
          Break;
        End;
  except
    Result := '';
  end;
end;


{------------------------------------------------------------------------------}

procedure TJZip.UnCompressSingleFile(input : TFileName; OutDir : String; ExtrFile : TFileName);
begin
  _ZipMstr.ZipFileName := input;
  _ZipMstr.ExtrBaseDir := IncludeTrailingPathDelimiter(OutDir);

  SetCurrentDir(_ZipMstr.ExtrBaseDir);

  _ZipMstr.ExtrOptions := _ZipMstr.ExtrOptions+[ExtrOverwrite];
  _ZipMstr.ExtrOptions := _ZipMstr.ExtrOptions-[ExtrDirNames];

  _ZipMstr.FSpecArgs.Clear;
  _ZipMstr.FSpecArgs.Add(ExtrFile);
  if _ZipMstr.Extract <> 0 then
    EJCompressException.Create('Zip Error: ' + _ZipMstr.Message);

  _ZipMstr.FSpecArgs.Clear;
  _ZipMstr.ZipFileName := '';
end;

{------------------------------------------------------------------------------}

Function  TJZip.VerifyFile(Z_File : TFileName) : Boolean;
begin
  _ZipMstr.ZipFileName := Z_File;
  // If the zip file is invalid, then the last error message will
  // be a 11036 message.
  if _ZipMstr.ErrCode = 11036 then
    Result := False
  else
    Result := True;

  _ZipMstr.ZipFileName := '';
end;

end.
